@@ -37,16 +37,21 @@
 #include <QStyleHints>
 
 #include "file-info.h"
+#include "file-info-job.h"
 #include "file-item-proxy-filter-sort-model.h"
 #include "file-item.h"
+#include "file-utils.h"
 
 #include <QDebug>
+#include <QTextLayout>
 
 using namespace Peony;
 using namespace Peony::DirectoryView;
 
 IconViewIndexWidget::IconViewIndexWidget(const IconViewDelegate *delegate, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget *parent) : QWidget(parent)
 {
+    installEventFilter(parent);
+
     setMouseTracking(true);
 
     m_edit_trigger.setInterval(3000);
@@ -139,13 +144,31 @@ IconViewIndexWidget::~IconViewIndexWidget()
     delete m_edit;
 }
 
+bool IconViewIndexWidget::eventFilter(QObject *watched, QEvent *event)
+{
+    if (event->type() == QEvent::Resize) {
+        adjustPos();
+        update();
+    }
+    return false;
+}
+
 void IconViewIndexWidget::paintEvent(QPaintEvent *e)
 {
+    IconView *view = m_delegate->getView();
+    auto expectedRect = view->visualRect(m_index);
+    auto rectPos = this->mapToParent(QPoint(0, 0));
+    if (expectedRect.topLeft() != rectPos) {
+        adjustPos();
+        update();
+        return;
+    }
+
     QWidget::paintEvent(e);
     QPainter p(this);
     //p.fillRect(0, 0, 999, 999, Qt::red);
 
-    adjustPos();
+    //adjustPos();
 
     //qDebug()<<m_option.backgroundBrush;
     //qDebug()<<this->size() << m_delegate->getView()->iconSize();
@@ -165,7 +188,14 @@ void IconViewIndexWidget::paintEvent(QPaintEvent *e)
     opt.rect = rawRect;
     auto tmp = opt.text;
     opt.text = nullptr;
+    //bug#99340,修改图标选中状态，会变暗
+    auto state = opt.state;
+    if((opt.state & QStyle::State_Enabled) && (opt.state & QStyle::State_Selected))
+    {
+        opt.state &= ~QStyle::State_Selected;
+    }
     QApplication::style()->drawControl(QStyle::CE_ItemViewItem, &opt, &p, opt.widget);
+    opt.state = state;
     if (b_elide_text)
     {
         int  charWidth = opt.fontMetrics.averageCharWidth();
@@ -173,50 +203,102 @@ void IconViewIndexWidget::paintEvent(QPaintEvent *e)
     }
     opt.text = std::move(tmp);
 
-    //auto textRectF = QRectF(0, m_delegate->getView()->iconSize().height(), this->width(), this->height());
-    p.save();
-
-    p.setPen(opt.palette.highlightedText().color());
-    p.translate(0, m_delegate->getView()->iconSize().height() + 5);
-    IconViewTextHelper::paintText(&p, opt, m_index, 9999, 2, 4);
-
-//    p.translate(-1, m_delegate->getView()->iconSize().height() + 13);
-//    //m_edit->document()->drawContents(&p);
-//    QTextOption textOption(Qt::AlignTop|Qt::AlignHCenter);
-//    textOption.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
-//    p.setFont(opt.font);
-//    p.setPen(opt.palette.highlightedText().color());
-//    p.drawText(QRect(1, 0, this->width() - 1, 9999), opt.text, textOption);
-    p.restore();
-
     //extra emblems
     if (!m_info.lock()) {
         return;
     }
     auto info = m_info.lock();
 
-    // draw color symbols
     auto colors = info->getColors();
-    int offset = 0;
-    const int MAX_LABEL_NUM = 3;
-    int startIndex = (colors.count() > MAX_LABEL_NUM ? colors.count() - MAX_LABEL_NUM : 0);
-    for (int i = startIndex; i < colors.count(); ++i) {
-        auto color = colors.at(i);
-        p.save();
-        p.setRenderHint(QPainter::Antialiasing);
-        p.translate(2, 2);
-        p.setPen(opt.palette.highlightedText().color());
-        p.setBrush(color);
-        p.drawEllipse(QRectF(offset, 0, 10, 10));
-        p.restore();
-        offset += 10/2;
+    auto lineSpacing = opt.fontMetrics.lineSpacing();
+    int yoffset = 0;
+    int iLine = 0;
+
+    // draw color symbols
+    if(info->uri().startsWith("favorite://")){/* 快速访问须特殊处理 */
+        //快速访问目录，颜色标记设置后更新不及时问题单独处理,修复bug#118015
+        auto matchInfo = FileInfo::fromUri(FileUtils::getEncodedUri(FileUtils::getTargetUri(info->uri())));
+        colors = matchInfo->getColors();
     }
 
+    if(0 < colors.count())
+    {
+        const int MAX_LABEL_NUM = 3;
+        int startIndex = (colors.count() > MAX_LABEL_NUM ? colors.count() - MAX_LABEL_NUM : 0);
+        int num =  colors.count() - startIndex;
+
+        QString text = opt.text;
+        QFont font = opt.font;
+        QTextLayout textLayout(text, font);
+
+        QTextOption textOpt;
+        textOpt.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
+
+        textLayout.setTextOption(textOpt);
+        textLayout.beginLayout();
+
+
+        QTextLine line = textLayout.createLine();
+        if (!line.isValid())
+            return;
+        int width = opt.rect.width() - (num+1)*6 - 2*2 - 4;
+        line.setLineWidth(width);
+
+        int xoffset = (width - line.naturalTextWidth())/2 ;
+        if(xoffset < 0)
+        {
+            xoffset = 2;
+        }
+        yoffset = (lineSpacing -12 )/2+2;
+
+        for (int i = startIndex; i < colors.count(); ++i) {
+            auto color = colors.at(i);
+            p.save();
+            p.setRenderHint(QPainter::Antialiasing);
+            p.translate(0, m_delegate->getView()->iconSize().height() + 5);
+
+           // p.translate(2, 2);
+            p.setPen(opt.palette.highlightedText().color());
+            p.setBrush(color);
+            p.drawEllipse(QRectF(xoffset, yoffset, 12, 12));
+            p.restore();
+            xoffset += 12/2;
+        }
+
+        yoffset = 0;
+        p.save();
+        p.translate(xoffset+10, m_delegate->getView()->iconSize().height() + 5);
+        p.setPen(opt.palette.highlightedText().color());
+
+        line.draw(&p, QPoint(0, yoffset));
+        yoffset += lineSpacing;
+        opt.text = text.mid(line.textLength());
+        textLayout.endLayout();
+        int heigth = opt.rect.height();
+        auto textSize = IconViewTextHelper::getTextSizeForIndex(opt, m_index, 2, 3);
+        int textHeigth = heigth - yoffset - m_delegate->getView()->iconSize().height() - 5;
+        if(textHeigth < textSize.height())
+        {
+           setFixedHeight(heigth+lineSpacing);
+        }
+        p.restore();
+        iLine++;
+    }
+    if(!opt.text.isEmpty())
+    {
+        p.save();
+        p.translate(0, m_delegate->getView()->iconSize().height() + 5 + yoffset);
+        p.setPen(opt.palette.highlightedText().color());
+        IconViewTextHelper::paintText(&p, opt, m_index, 9999, 2, 4-iLine);
+        p.restore();
+    }
     //paint symbolic link emblems
     if (info->isSymbolLink()) {
         QIcon icon = QIcon::fromTheme("emblem-symbolic-link");
         //qDebug()<< "symbolic:" << info->symbolicIconName();
-        icon.paint(&p, this->width() - 30, 10, 20, 20, Qt::AlignCenter);
+        //icon.paint(&p, this->width() - 30, 10, 20, 20, Qt::AlignCenter);
+        //Adjust link emblem to topLeft.link story#8354
+        icon.paint(&p, this->rect().x() + 10, m_delegate->getView()->iconSize().height() - 10, 20, 20, Qt::AlignCenter);
     }
 
     //paint access emblems
@@ -292,7 +374,7 @@ void IconViewIndexWidget::mouseDoubleClickEvent(QMouseEvent *event)
 void IconViewIndexWidget::adjustPos()
 {
     IconView *view = m_delegate->getView();
-    if (m_index.model() != view->model())
+//    if (m_index.model() != view->model())
         return;
 
     if (!view->selectionModel()->selectedIndexes().contains(m_index)) {
